//+------------------------------------------------------------------+
//|                                               FX5_Divergence.mq4 |
//|                                                              FX5 |
//|                                                    hazem@uk2.net |
//| DiNapolli and DEMA additions by mladen                           |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, FX5"
#property link      "hazem@uk2.net"
//----
#property indicator_separate_window
#property indicator_buffers 7
#property indicator_color1 Gold
#property indicator_color2 Gold
#property indicator_color3 FireBrick
#property indicator_color4 FireBrick
#property indicator_color5 Green
#property indicator_color6 Red
#property indicator_color7 Red
#property indicator_width1 2
#property indicator_width3 2

//---- input parameters
extern string separator1="*** MACD/OSMA Settings ***";
extern double fastEMA=12;
extern int    fastMode = MODE_SMA;
extern double slowEMA=26;
extern int    slowMode = MODE_EMA;
extern double signal=9;
extern int    signalMode = MODE_SMA;
extern bool   UseOSMA = false;
extern string separator2="*** Indicator Settings";
extern double positiveSensitivity=0.0001;
extern double negativeSensitivity=-0.0001;
extern double historyBarsCount=0;
extern bool   drawDivergenceLines=true;
extern bool   divergenceAlert           = true;
extern bool   divergenceAlertsMessage   = true;
extern bool   divergenceAlertsSound     = true;
extern bool   divergenceAlertsEmail     = false;
extern bool   divergenceAlertsNotify    = false;
extern string divergenceAlertsSoundName = "alert1.wav";

//---- buffers
double upOsMA[];
double upOsMAd[];
double downOsMA[];
double downOsMAu[];
double bullishDivergence[];
double bearishDivergence[];
double OsMA[];
static int chartBarsCount;
string shortName;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   SetIndexBuffer(0, upOsMA);            SetIndexStyle(0, DRAW_HISTOGRAM);
   SetIndexBuffer(1, upOsMAd);           SetIndexStyle(1, DRAW_HISTOGRAM);
   SetIndexBuffer(2, downOsMA);          SetIndexStyle(2, DRAW_HISTOGRAM);
   SetIndexBuffer(3, downOsMAu);         SetIndexStyle(3, DRAW_HISTOGRAM);
   SetIndexBuffer(4, bullishDivergence); SetIndexStyle(4, DRAW_ARROW); SetIndexArrow(4, 233);
   SetIndexBuffer(5, bearishDivergence); SetIndexStyle(5, DRAW_ARROW); SetIndexArrow(5, 234);
   SetIndexBuffer(6, OsMA);
   IndicatorDigits(MarketInfo(Symbol(), MODE_DIGITS) + 2);
   shortName = "FX5_Divergence("+DoubleToStr(fastEMA,4)+ ","+DoubleToStr(slowEMA,4)+","+DoubleToStr(signal,4)+")";
   IndicatorShortName(shortName);
//----
   if(historyBarsCount<=0 || historyBarsCount > Bars)
      chartBarsCount=Bars;
   else
      chartBarsCount=historyBarsCount;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   for(int i=ObjectsTotal() - 1; i>=0; i--)
     {
      string label=ObjectName(i);
      if(StringSubstr(label, 0, 14)!="DivergenceLine")
         continue;
      ObjectDelete(label);
     }
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int countedBars=IndicatorCounted();
   if(countedBars < 0)
      countedBars=0;
//----
   CalculateOsMA(countedBars);
   CalculateDivergence(countedBars);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CalculateDivergence(int countedBars)
  {
   double arrowSeparation=1/MathPow(10, MarketInfo(Symbol(),
                            MODE_DIGITS) + 2) * 50;
//----    
   for(int i=chartBarsCount - countedBars; i>=0; i--)
     {
      bearishDivergence[i]=EMPTY_VALUE;
      bullishDivergence[i]=EMPTY_VALUE;
      //----   
      int firstPeakOrTroughShift=GetFirstPeakOrTrough(i);
      double firstPeakOrTroughOsMA=OsMA[firstPeakOrTroughShift];
      if(firstPeakOrTroughOsMA > 0)
        {
         int peak_0  =GetIndicatorLastPeak(i);
         int trough_0=GetIndicatorLastTrough(peak_0);
         int peak_1  =GetIndicatorLastPeak(trough_0);
         int trough_1=GetIndicatorLastTrough(peak_1);
        }
      else
        {
         trough_0=GetIndicatorLastTrough(i);
         peak_0  =GetIndicatorLastPeak(trough_0);
         trough_1=GetIndicatorLastTrough(peak_0);
         peak_1  =GetIndicatorLastPeak(trough_1);
        }
      //----  
      if(peak_0==-1 || peak_1==-1 || trough_0==-1 ||
         trough_1==-1)
         continue;
      //----          
      double indicatorLastPeak=OsMA[peak_0];
      double indicatorThePeakBefore=OsMA[peak_1];
      double indicatorLastTrough=OsMA[trough_0];
      double indicatorTheTroughBefore=OsMA[trough_1];
/*      
       int pricePeak_0 = GetPriceLastPeak(peak_0);
       int pricePeak_1 = GetPriceLastPeak(peak_1);
       int priceTrough_0 = GetPriceLastTrough(trough_0);
       int priceTrough_1 = GetPriceLastTrough(trough_1);
*/
      int pricePeak_0=peak_0;
      int pricePeak_1=peak_1;
      int priceTrough_0=trough_0;
      int priceTrough_1=trough_1;
      //----
      double priceLastPeak=High[pricePeak_0];
      double priceThePeakBefore=High[pricePeak_1];
      double priceLastTrough=Low[priceTrough_0];
      double priceTheTroughBefore=Low[priceTrough_1];
      // Bearish divergence condition         
      if((priceLastPeak > priceThePeakBefore &&
          indicatorLastPeak < indicatorThePeakBefore) ||
          (priceLastPeak < priceThePeakBefore &&
          indicatorLastPeak > indicatorThePeakBefore))
        {
         bearishDivergence[peak_0]=upOsMA[peak_0] + arrowSeparation;
         if(drawDivergenceLines)
           {
            PriceDrawLine(Time[pricePeak_0], Time[pricePeak_1],
                          priceLastPeak, priceThePeakBefore, Red);
            IndicatorDrawLine(Time[peak_0], Time[peak_1], indicatorLastPeak,
                              indicatorThePeakBefore, Red);
            if (divergenceAlert) DisplayAlert("Bearish divergence",peak_0); 
           }
         continue;
        }
      // Bullish divergence condition           
      if((priceLastTrough < priceTheTroughBefore &&
          indicatorLastTrough > indicatorTheTroughBefore) ||
          (priceLastTrough > priceTheTroughBefore &&
          indicatorLastTrough < indicatorTheTroughBefore))
        {
         bullishDivergence[trough_0]=downOsMA[trough_0] - arrowSeparation;
         if(drawDivergenceLines)
           {
            PriceDrawLine(Time[priceTrough_0], Time[priceTrough_1],
                          priceLastTrough, priceTheTroughBefore, Green);
            IndicatorDrawLine(Time[trough_0], Time[trough_1], indicatorLastTrough,
                              indicatorTheTroughBefore, Green);
            if (divergenceAlert) DisplayAlert("Bullish divergence",trough_0);  
           }
         continue;
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

void CalculateOsMA(int countedBars)
{
   for(int i=Bars - countedBars; i>=0; i--)
   {
      double MACD = iCustom(NULL,0,"AllMACD_v2.1",0,PRICE_CLOSE,fastEMA,fastMode,PRICE_CLOSE,slowEMA,slowMode,signal,signalMode,1,4,i);
      if (UseOSMA)
            OsMA[i] = iCustom(NULL,0,"AllMACD_v2.1",0,PRICE_CLOSE,fastEMA,fastMode,PRICE_CLOSE,slowEMA,slowMode,signal,signalMode,1,6,i);
      else  OsMA[i] = MACD;
      
      //
      //
      //
      //
      //
            
      upOsMA[i]   = EMPTY_VALUE;
      upOsMAd[i]  = EMPTY_VALUE;
      downOsMA[i] = EMPTY_VALUE;
      downOsMAu[i]= EMPTY_VALUE;
      if(OsMA[i] > 0)
         if (OsMA[i]>OsMA[i+1])
               upOsMA[i]  = OsMA[i];
         else  upOsMAd[i] = OsMA[i];
      if(OsMA[i] < 0)
         if (OsMA[i]<OsMA[i+1])
               downOsMA[i]  = OsMA[i];
         else  downOsMAu[i] = OsMA[i];
   }
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetPositiveRegionStart(int index)
  {
   for(int i=index + 1; i < Bars; i++)
     {
      if(OsMA[i]>=OsMA[i-1] && OsMA[i]>=OsMA[i+1] &&
         OsMA[i]>=OsMA[i+2] && OsMA[i] > positiveSensitivity)
         return(i);
     }
   return(-1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetNegativeRegionStart(int index)
  {
   for(int i=index + 1; i < Bars; i++)
     {
      if(OsMA[i]<=OsMA[i-1] && OsMA[i]<=OsMA[i+1] &&
         OsMA[i]<=OsMA[i+2] && OsMA[i] < negativeSensitivity)
         return(i);
     }
   return(-1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetFirstPeakOrTrough(int index)
  {
   for(int i=index + 1; i < Bars; i++)
     {
      if((OsMA[i]>=OsMA[i-1] && OsMA[i]>=OsMA[i+1] &&
         OsMA[i]>=OsMA[i+2] && OsMA[i] > positiveSensitivity) ||
         (OsMA[i]<=OsMA[i-1] && OsMA[i]<=OsMA[i+1] &&
         OsMA[i]<=OsMA[i+2] && OsMA[i] < negativeSensitivity))
         return(i);
     }
   return(-1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetIndicatorLastPeak(int index)
  {
   int regionStart=GetPositiveRegionStart(index);
   if(regionStart==-1)
      return(-1);
//----   
   int peakShift=0;
   double peakValue=0;
//----
   for(int i=regionStart; i < Bars; i++)
     {
      if(OsMA[i] > peakValue && OsMA[i]>=OsMA[i-1] &&
         OsMA[i]>=OsMA[i+1] && OsMA[i]>=OsMA[i+2] &&
         OsMA[i] > positiveSensitivity)
        {
         peakValue=OsMA[i];
         peakShift=i;
        }
      if(OsMA[i] < 0)
         break;
     }
   return(peakShift);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetIndicatorLastTrough(int index)
  {
   int regionStart=GetNegativeRegionStart(index);
   if(regionStart==-1)
      return(-1);
//----         
   int troughShift=0;
   double troughValue=0;
//----   
   for(int i=regionStart; i < Bars; i++)
     {
      if(OsMA[i] < troughValue && OsMA[i]<=OsMA[i-1] &&
         OsMA[i]<=OsMA[i+1] && OsMA[i]<=OsMA[i+2] &&
         OsMA[i] < negativeSensitivity)
        {
         troughValue=OsMA[i];
         troughShift=i;
        }
      if(OsMA[i] > 0)
         break;
     }
   return(troughShift);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void PriceDrawLine(datetime x1, datetime x2, double y1, double y2,
                   color lineColor)
  {
   string label="DivergenceLine# " + DoubleToStr(x1, 0);
   ObjectDelete(label);
   ObjectCreate(label, OBJ_TREND, 0, x1, y1, x2, y2, 0, 0);
   ObjectSet(label, OBJPROP_RAY, 0);
   ObjectSet(label, OBJPROP_COLOR, lineColor);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void IndicatorDrawLine(datetime x1, datetime x2, double y1, double y2,
                       color lineColor)
  {
   int indicatorWindow=WindowFind(shortName);
   if(indicatorWindow < 0)
      return;
//----      
   string label="DivergenceLine$# " + DoubleToStr(x1, 0);
   ObjectDelete(label);
   ObjectCreate(label, OBJ_TREND, indicatorWindow, x1, y1, x2, y2, 0, 0);
   ObjectSet(label, OBJPROP_RAY, 0);
   ObjectSet(label, OBJPROP_COLOR, lineColor);
  }
//+------------------------------------------------------------------+
//
//
//
//
//

void DisplayAlert(string doWhat, int shift)
{
    string dmessage;
    static datetime lastAlertTime;
    if(shift <= 2 && Time[0] != lastAlertTime)
    {
      dmessage =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," FX5 MACD Divergence ",doWhat);
          if (divergenceAlertsMessage) Alert(dmessage);
          if (divergenceAlertsNotify)  SendNotification(dmessage);
          if (divergenceAlertsEmail)   SendMail(StringConcatenate(Symbol()," FX5 MACD Divergence "),dmessage);
          if (divergenceAlertsSound)   PlaySound(divergenceAlertsSoundName); 
          lastAlertTime = Time[0];
    }
}
